#!/usr/bin/perl -w
# This script takes include files for Makefiles and converts their 
# definitions into a sed script.

use strict;
use Data::Dumper;

#my $INCLUDES="basic owsproxy";
# This script receives the list of *.include Makefiles as a single argument string.
my $INCLUDES=$ARGV[0];

%main::rtable=();

# This saves us from explicitely define the variable expansions with sed
#   First the gathering of variables to be looked for
foreach my $include_prefix (split(" ", $INCLUDES))
{
	&gather_replacements($include_prefix);
}
open (SED, ">", "replace.sed");
foreach my $var (keys %main::rtable)
{
	print SED ("s|\@$var\@|".$main::rtable{$var}."|g;\n");
}
close(SED);

open (M4, ">", "replace.m4");
foreach my $var (keys %main::rtable)
{
	print M4 ("define(`\@$var\@', `".$main::rtable{$var}."\')dnl\n");
}
close(M4);

sub gather_replacements
{
	my $includefile=shift;
	my ($line, $var, $val);

	open(INCL, "<", $includefile) || die "$! $includefile could not be opened.\n";
	while(defined($line=<INCL>))
	{
		# Ignore comment lines
		next if $line =~ /\s*#/ ;
		if($line =~ /\s*(\w+)+\s*=\s*(.*)/)
		{
			$var=$1;
			$val=$2;
			$main::rtable{$var}=$val;
		}
	}
	while(&recursive_expand() != 0)
	{}
}

sub recursive_expand
{
	my $changes=0;
	my($var, $val, $unexvar, $mkfvar, $mkfexpr, $exp, $pattern);
	foreach $var (keys %main::rtable)
	{
		if($main::rtable{$var} =~ /(\$\((\w+)\))/)
		{
			$mkfexpr=$1;
			$mkfvar=$2;
			$pattern=quotemeta($mkfexpr);
			if(defined($main::rtable{$mkfvar}))
			{
#				print "Now we can replace $mkfexpr by ".$main::rtable{$mkfvar}."!\n";
				$main::rtable{$var}=~ s/$pattern/$main::rtable{$mkfvar}/gis;
#				print "New value: ".$main::rtable{$var}."\n";
				$changes++;
			}
			elsif(defined($ENV{$mkfvar}))
			{
#				print "Now we can replace $mkfexpr by Environment variable ".$ENV{$mkfvar}."!\n";
				$main::rtable{$var}=~ s/$pattern/$ENV{$mkfvar}/gis;
#				print "New value: ".$main::rtable{$var}."\n";
				$changes++;
			}
		}
	}
	return $changes;
}


__END__
$REPLACE_VARS=`cat "$TEMPLATE_HOME"*.include | egrep -v ^#   | awk '{print$1}' | sort | uniq | xargs echo`;
#echo $REPLACE_VARS
#   Building an extended regular expression to be used by egrep later
REPLACE_RE=`echo $REPLACE_VARS | sed 's/ /|/g'`
#   Identifying the files to be used by the template definitions (excluding certain directories)
REPLACE_FILES=`egrep -I "@("$REPLACE_RE")@" -r $INSTALL_SOURCE_DIR | egrep -v "\/(\.svn|rpm|debian)\/" | sed 's/:.*$//' | sort | uniq`

#echo "Files to be rewritten: $REPLACE_FILES"

for t in "$REPLACE_FILES"
do
	CLEAN_FILENAME=`echo $t | sed -e "s|$INSTALL_SOURCE_DIR||;"`
	CMD=""
	for r in $REPLACE_VARS
	do
#		echo $t $r
		SEDCMD="s/@$r@/\$("$r")/g;"
		CMD=$CMD"$SEDCMD"
	done
	echo sed -e "'$CMD' < $t > "$RPM_BUILD_ROOT$TC_TARGET$CLEAN_FILENAME ; 
done

echo $ALL_COMMANDS
